@@ -270,6 +270,18 @@
 		  (org-list-demote-modify-bullet '(("1." . "+"))))
 	      (org-indent-item))
 	    (buffer-string))))
+  (should
+   (equal "
+a. Item 1
+   - Item 2"
+	  (org-test-with-temp-text "
+a. Item 1
+b. Item 2<point>"
+	    (let ((org-plain-list-ordered-item-terminator t)
+		  (org-list-allow-alphabetical t)
+		  (org-list-demote-modify-bullet '(("A." . "a.") ("a." . "-"))))
+	      (org-indent-item))
+	    (buffer-string))))
   ;; When a region is selected, indent every item within.
   (should
    (equal "
@@ -595,9 +607,11 @@
     (should-error (org-move-item-up)))
   ;; ... unless `org-list-use-circular-motion' is non-nil.  In this
   ;; case, move to the first item.
-  (org-test-with-temp-text "- item 1\n- item 2\n- item 3"
-    (let ((org-list-use-circular-motion t)) (org-move-item-up))
-    (should (equal (buffer-string) "- item 2\n- item 3\n- item 1")))
+  (should
+   (equal "- item 2\n- item 3\n- item 1"
+	  (org-test-with-temp-text "- item 1\n- item 2\n- item 3"
+	    (let ((org-list-use-circular-motion t)) (org-move-item-up))
+	    (buffer-string))))
   ;; Preserve item visibility.
   (org-test-with-temp-text "* Headline\n- item 1\n  body 1\n- item 2\n  body 2"
     (let ((org-cycle-include-plain-lists t))
@@ -735,6 +749,18 @@
 	  (org-test-with-temp-text "- <point>item"
 	    (org-insert-item)
 	    (buffer-string))))
+  ;; When called at the very end of the list, insert new item as
+  ;; a sibling of the very last one.
+  (should
+   (equal "- A\n\n  - B\n\n  - "
+	  (org-test-with-temp-text "- A\n\n - B\n\n<point>"
+	    (org-insert-item)
+	    (buffer-string))))
+  (should
+   (equal "- A\n\n  - B\n\n  - "
+	  (org-test-with-temp-text "- A\n\n  - B\n\n  <point>"
+	    (org-insert-item)
+	    (buffer-string))))
   ;; When called on tag in a descriptive list, insert new item before
   ;; current one too.
   (should
@@ -761,6 +787,12 @@
 	    (let ((org-M-RET-may-split-line  '((default . nil))))
 	      (org-insert-item))
 	    (buffer-string))))
+  ;; Re-order automatically.
+  (should
+   (equal "1. A\n\n2. \n\n3. \n\n4. B"
+	  (org-test-with-temp-text "1. A<point>\n\n2. \n\n3. B"
+	    (org-insert-item)
+	    (buffer-string))))
   ;; Preserve list visibility when inserting an item.
   (should
    (equal
@@ -772,7 +804,20 @@
 	(org-cycle)
 	(org-insert-item)
 	(list (get-char-property (line-beginning-position 0) 'invisible)
-	      (get-char-property (line-end-position 2) 'invisible)))))))
+	      (get-char-property (line-end-position 2) 'invisible))))))
+  ;; Test insertion in area after a sub-list.  In particular, if point
+  ;; is right at the end of the previous sub-list, still insert
+  ;; a sub-item in that list.
+  (should
+   (= 2
+      (org-test-with-temp-text "- item\n  - sub-list\n<point>  resume item"
+	(org-insert-item)
+	(current-indentation))))
+  (should
+   (= 0
+      (org-test-with-temp-text "- item\n  - sub-list\n  resume item<point>"
+	(org-insert-item)
+	(current-indentation)))))
 
 (ert-deftest test-org-list/repair ()
   "Test `org-list-repair' specifications."
